@@ -44,6 +44,10 @@ except ImportError:
     import simplejson as json
 
 
+ENTER_BOLD_MODE = "\x1b[1m"
+EXIT_ATTRIBUTE_MODE = "\x1b[m"
+ENTER_UNDERLINE_MODE = "\x1b[4m"
+
 # Disable CLI web browsers
 term = os.environ.pop("TERM", None)
 # This import must be done with an empty $TERM, otherwise a command-line browser may be started
@@ -82,6 +86,11 @@ def is_wsl():
     return False
 
 
+def is_windows():
+    """Return whether we are running under the Windows"""
+    return sys.platform.startswith("win")
+
+
 def is_sailfish_os():
     """Return whether we are running on Sailfish OS"""
     if "linux" in platform.system().lower() and os.access(
@@ -108,7 +117,7 @@ def is_chromeos_garcon():
     # https://source.chromium.org/search?q=garcon-url-handler
     try:
         return "garcon-url-handler" in webbrowser.get().name
-    except AttributeError:
+    except:
         return False
 
 
@@ -218,46 +227,78 @@ def parse_color(color_str):
     comps = color_str.split(" ")
     color = ""
     background_color = ""
-    bold, underline, italics, dim, reverse = False, False, False, False, False
-    for comp in comps:
+    underline_color = ""
+    bold = False
+    underline = False
+    italics = False
+    dim = False
+    reverse = False
+    i = 0
+    while i < len(comps):
+        comp = comps[i]
         # Remove quotes
         comp = comp.strip("'\" ")
         if comp == "--bold" or comp == "-o":
             bold = True
         elif comp == "--underline" or comp == "-u":
-            underline = True
+            underline = "single"
+        elif comp.startswith("--underline="):
+            # TODO(python>3.8): use removeprefix
+            underline = comp[len("--underline=") :]
+        elif comp.startswith(
+            "-u"
+        ):  # Multiple short options like "-rucurly" are not yet supported.
+            # TODO(python>3.8): use removeprefix
+            underline = comp[len("-u") :]
         elif comp == "--italics" or comp == "-i":
             italics = True
         elif comp == "--dim" or comp == "-d":
             dim = True
         elif comp == "--reverse" or comp == "-r":
             reverse = True
-        elif comp.startswith("--background"):
-            # Background color
-            c = comp[len("--background=") :]
-            parsed_c = parse_one_color(c)
-            # We prefer the unparsed version - if it says "brgreen", we use brgreen,
-            # instead of 00ff00
-            if better_color(background_color, parsed_c) == parsed_c:
-                background_color = c
-        elif comp.startswith("-b"):
-            # Background color in short.
-            skip = len("-b")
-            if comp[len("-b=")] in ["=", " "]:
-                skip += 1
-            c = comp[skip:]
-            parsed_c = parse_one_color(c)
-            if better_color(background_color, parsed_c) == parsed_c:
-                background_color = c
         else:
-            # Regular color
-            parsed_c = parse_one_color(comp)
-            if better_color(color, parsed_c) == parsed_c:
-                color = comp
+
+            def parse_opt(
+                current_best: str, i: int, long_opt: str, short_opt: str | None
+            ) -> str:
+                if comp.startswith(long_opt):
+                    c = comp[len(long_opt) :]
+                    parsed_c = parse_one_color(c)
+                    # We prefer the unparsed version - if it says "brgreen", we use brgreen,
+                    # instead of 00ff00
+                    if better_color(current_best, parsed_c) == parsed_c:
+                        return True, c, i
+                elif short_opt is not None and comp.startswith(short_opt):
+                    if comp == short_opt:
+                        if i + 1 == len(comps):
+                            c = ""
+                        else:
+                            c = comps[i + 1]
+                            i += 1
+                    else:
+                        c = comp[len(short_opt) :]
+                    parsed_c = parse_one_color(c)
+                    if better_color(current_best, parsed_c) == parsed_c:
+                        return True, c, i
+                return False, current_best, i
+
+            is_bg, background_color, i = parse_opt(
+                background_color, i, "--background", "-b"
+            )
+            is_ul, underline_color, i = parse_opt(
+                underline_color, i, "--underline-color", None
+            )
+            if not (is_bg or is_ul):
+                # Regular color
+                parsed_c = parse_one_color(comp)
+                if better_color(color, parsed_c) == parsed_c:
+                    color = comp
+        i += 1
 
     return {
         "color": color,
         "background": background_color,
+        "underline-color": underline_color,
         "bold": bold,
         "underline": underline,
         "italics": italics,
@@ -275,8 +316,8 @@ def unparse_color(col):
         ret += col["color"]
     if col["bold"]:
         ret += " --bold"
-    if col["underline"]:
-        ret += " --underline"
+    if col["underline"] is not None:
+        ret += " --underline=" + str(col["underline"])
     if col["italics"]:
         ret += " --italics"
     if col["dim"]:
@@ -285,6 +326,8 @@ def unparse_color(col):
         ret += " --reverse"
     if col["background"]:
         ret += " --background=" + col["background"]
+    if col["underline-color"]:
+        ret += " --underline-color=" + col["underline-color"]
     return ret
 
 
@@ -562,47 +605,6 @@ def html_color_for_ansi_color_index(val):
         return arr[val]
 
 
-# Function to return special ANSI escapes like exit_attribute_mode
-g_special_escapes_dict = None
-
-
-def get_special_ansi_escapes():
-    global g_special_escapes_dict
-    if g_special_escapes_dict is None:
-        try:
-            import curses
-
-            g_special_escapes_dict = {}
-            curses.setupterm()
-
-            # Helper function to get a value for a tparm
-            def get_tparm(key):
-                val = None
-                key = curses.tigetstr(key)
-                if key:
-                    val = curses.tparm(key)
-                if val:
-                    val = val.decode("utf-8")
-                return val
-
-            # Just a few for now
-            g_special_escapes_dict["exit_attribute_mode"] = get_tparm("sgr0")
-            g_special_escapes_dict["bold"] = get_tparm("bold")
-            g_special_escapes_dict["underline"] = get_tparm("smul")
-        except ImportError:
-            print("WARNING: The python curses module is missing.")
-            print("WARNING: Falling back to xterm-256color settings.")
-            print("WARNING: Rebuild python with curses headers!")
-
-            g_special_escapes_dict = {
-                "exit_attribute_mode": "\x1b(B\x1b[m",
-                "bold": "\x1b[1m",
-                "underline": "\x1b[4m",
-            }
-
-    return g_special_escapes_dict
-
-
 # Given a known ANSI escape sequence, convert it to HTML and append to the list
 # Returns whether we have an open <span>
 
@@ -647,8 +649,7 @@ def append_html_for_ansi_escape(full_val, result, span_open):
         return True  # span now open
 
     # Try special escapes
-    special_escapes = get_special_ansi_escapes()
-    if full_val == special_escapes["exit_attribute_mode"]:
+    if full_val == EXIT_ATTRIBUTE_MODE:
         close_span()
         return False
 
@@ -774,169 +775,6 @@ class FishBinding:
         }
 
 
-class BindingParser:
-    """Class to parse codes for bind command"""
-
-    # TODO: What does snext and sprevious mean ?
-    readable_keys = {
-        "dc": "Delete",
-        "npage": "Page Up",
-        "ppage": "Page Down",
-        "sdc": "Shift Delete",
-        "shome": "Shift Home",
-        "left": "Left Arrow",
-        "right": "Right Arrow",
-        "up": "Up Arrow",
-        "down": "Down Arrow",
-        "sleft": "Shift Left",
-        "sright": "Shift Right",
-        "btab": "Shift Tab",
-    }
-
-    def set_buffer(self, buffer):
-        """Sets code to parse"""
-
-        self.buffer = buffer or b""
-        self.index = 0
-
-    def get_char(self):
-        """Gets next character from buffer"""
-        if self.index >= len(self.buffer):
-            return "\0"
-        c = self.buffer[self.index]
-        self.index += 1
-        return c
-
-    def unget_char(self):
-        """Goes back by one character for parsing"""
-
-        self.index -= 1
-
-    def end(self):
-        """Returns true if reached end of buffer"""
-
-        return self.index >= len(self.buffer)
-
-    def parse_control_sequence(self):
-        """Parses terminal specifiec control sequences"""
-
-        result = ""
-        c = self.get_char()
-
-        # \e0 is used to denote start of control sequence
-        if c == "O":
-            c = self.get_char()
-
-        # \[1\; is start of control sequence
-        if c == "1":
-            b = self.get_char()
-            c = self.get_char()
-            if b == "\\" and c == "~":
-                result += "Home"
-            elif c == ";":
-                c = self.get_char()
-
-        # 3 is Alt
-        if c == "3":
-            result += "ALT - "
-            c = self.get_char()
-
-        # \[4\~ is End
-        if c == "4":
-            b = self.get_char()
-            c = self.get_char()
-            if b == "\\" and c == "~":
-                result += "End"
-
-        # 5 is Ctrl
-        if c == "5":
-            result += "CTRL - "
-            c = self.get_char()
-
-        # 9 is Alt
-        if c == "9":
-            result += "ALT - "
-            c = self.get_char()
-
-        if c == "A":
-            result += "Up Arrow"
-        elif c == "B":
-            result += "Down Arrow"
-        elif c == "C":
-            result += "Right Arrow"
-        elif c == "D":
-            result += "Left Arrow"
-        elif c == "F":
-            result += "End"
-        elif c == "H":
-            result += "Home"
-
-        return result
-
-    def get_readable_binding(self):
-        """Gets a readable representation of binding"""
-
-        try:
-            result = BindingParser.readable_keys[self.buffer.lower()]
-        except KeyError:
-            result = self.parse_binding()
-
-        return result
-
-    def parse_binding(self):
-        readable_command = ""
-        result = ""
-        alt = ctrl = False
-
-        while not self.end():
-            c = self.get_char()
-
-            if c == "\\":
-                c = self.get_char()
-                if c == "e":
-                    d = self.get_char()
-                    if d == "O":
-                        self.unget_char()
-                        result += self.parse_control_sequence()
-                    elif d == "\\":
-                        if self.get_char() == "[":
-                            result += self.parse_control_sequence()
-                        else:
-                            self.unget_char()
-                            self.unget_char()
-                            alt = True
-                    elif d == "\0":
-                        result += "ESC"
-                    else:
-                        alt = True
-                        self.unget_char()
-                elif c == "c":
-                    ctrl = True
-                elif c == "n":
-                    result += "Enter"
-                elif c == "t":
-                    result += "Tab"
-                elif c == "b":
-                    result += "Backspace"
-                elif c.isalpha():
-                    result += "\\" + c
-                else:
-                    result += c
-            elif c == "\x7f":
-                result += "Backspace"
-            else:
-                result += c
-        if ctrl:
-            readable_command += "CTRL - "
-        if alt:
-            readable_command += "ALT - "
-
-        if result == "":
-            return "unknown-control-sequence"
-
-        return readable_command + result
-
-
 class FishConfigTCPServer(SocketServer.ThreadingMixIn, SocketServer.TCPServer):
     """TCPServer that only accepts connections from localhost (IPv4/IPv6)."""
 
@@ -1114,14 +952,13 @@ class FishConfigHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         # Put all the bindings into a list
         bindings = []
         command_to_binding = {}
-        binding_parser = BindingParser()
 
         for line in out.split("\n"):
             comps = line.split(" ", 2)
 
             # If we don't have "bind", a sequence and a mapping,
             # it's not a valid binding.
-            if len(comps) < 3:
+            if len(comps) != 3:
                 continue
 
             # Store the "--preset" value for later
@@ -1133,26 +970,20 @@ class FishConfigHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
                 preset = False
                 comps = line.split(" ", 3)[1:]
             # Check again if we removed the level.
-            if len(comps) < 3:
+            if len(comps) != 3:
                 continue
 
-            if comps[1] == "-k":
-                key_name, command = comps[2].split(" ", 1)
-                binding_parser.set_buffer(key_name.capitalize())
-            else:
-                key_name = None
-                command = comps[2]
-                binding_parser.set_buffer(comps[1])
+            key = comps[1]
+            command = comps[2]
 
             if command in bindings_blacklist:
                 continue
 
-            readable_binding = binding_parser.get_readable_binding()
             if command in command_to_binding:
                 fish_binding = command_to_binding[command]
-                fish_binding.add_binding(line, readable_binding)
+                fish_binding.add_binding(line, key)
             else:
-                fish_binding = FishBinding(command, line, readable_binding)
+                fish_binding = FishBinding(command, line, key)
                 bindings.append(fish_binding)
                 command_to_binding[command] = fish_binding
 
@@ -1508,7 +1339,7 @@ class FishConfigHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
                     "fish_pager_color_secondary_description",
                 )
             )
-            output=""
+            output = ""
             for item in postvars.get("colors"):
                 what = item.get("what")
                 color = item.get("color")
@@ -1583,9 +1414,13 @@ redirect_template_html = """
 # find fish
 fish_bin_dir = os.environ.get("__fish_bin_dir")
 fish_bin_path = None
+
+# only need the '.exe' extension on Windows
+fish_bin_name = "fish.exe" if is_windows() else "fish"
+
 if not fish_bin_dir:
     print("The $__fish_bin_dir environment variable is not set. " "Looking in $PATH...")
-    fish_bin_path = find_executable("fish")
+    fish_bin_path = find_executable(fish_bin_name)
     if not fish_bin_path:
         print("fish could not be found. Is fish installed correctly?")
         sys.exit(-1)
@@ -1593,7 +1428,7 @@ if not fish_bin_dir:
         print("fish found at '%s'" % fish_bin_path)
 
 else:
-    fish_bin_path = os.path.join(fish_bin_dir, "fish")
+    fish_bin_path = os.path.join(fish_bin_dir, fish_bin_name)
 
 if not os.access(fish_bin_path, os.X_OK):
     print(
@@ -1664,25 +1499,44 @@ url = "http://localhost:%d/%s/%s" % (PORT, authkey, initial_tab)
 # Create temporary file to hold redirect to real server. This prevents exposing
 # the URL containing the authentication key on the command line (see
 # CVE-2014-2914 or https://github.com/fish-shell/fish-shell/issues/1438).
-f = tempfile.NamedTemporaryFile(prefix="web_config", suffix=".html", mode="w")
+#
+
+# If on Windows, the file needs to be closed after writing, otherwise, the browser won't be able to open it."
+# unfortunately this was added in python 3.12, so we don't add it on other platforms
+# to support older python versions there.
+kwargs = {}
+if is_windows():
+    kwargs["delete_on_close"] = False
+f = tempfile.NamedTemporaryFile(
+    prefix="web_config",
+    suffix=".html",
+    mode="w",
+    delete=True,
+    **kwargs,
+)
 
 f.write(redirect_template_html % (url, url))
 f.flush()
 
+if is_windows():
+    f.close()
+
 # Open temporary file as URL
 # Use open on macOS >= 10.12.5 to work around #4035.
 fileurl = "file://" + f.name
 
-esc = get_special_ansi_escapes()
+if is_windows():
+    fileurl = fileurl.replace("\\", "/")
+
 print(
     "Web config started at %s%s%s"
-    % (esc["underline"], fileurl, esc["exit_attribute_mode"])
+    % (ENTER_UNDERLINE_MODE, fileurl, EXIT_ATTRIBUTE_MODE)
 )
 print(
     "If that doesn't work, try opening %s%s%s"
-    % (esc["underline"], url, esc["exit_attribute_mode"])
+    % (ENTER_UNDERLINE_MODE, url, EXIT_ATTRIBUTE_MODE)
 )
-print("%sHit ENTER to stop.%s" % (esc["bold"], esc["exit_attribute_mode"]))
+print("%sHit ENTER to stop.%s" % (ENTER_BOLD_MODE, EXIT_ATTRIBUTE_MODE))
 
 
 def runThing():
@@ -1717,13 +1571,55 @@ httpd.daemon_threads = True
 
 # Select on stdin and httpd
 stdin_no = sys.stdin.fileno()
-try:
+
+
+def create_socket(start_port, end_port):
+    """Attempt to create a socket from a range of ports."""
+    for port in range(start_port, end_port + 1):
+        try:
+            sck = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            sck.bind(("localhost", port))
+            sck.listen()
+            return (sck, port)
+        except socket.error:
+            pass
+    print("Unable to find an open port between {} and {}.".format(start_port, end_port))
+    sys.exit(-1)
+
+
+def capture_enter(port):
+    """Read keyboard events and establish a socket connection when pressing the Enter."""
+    import msvcrt
+
     while True:
-        ready_read = select.select([sys.stdin.fileno(), httpd.fileno()], [], [])
-        if ready_read[0][0] < 1:
+        if msvcrt.kbhit():
+            key = msvcrt.getch().decode()
+            if key == "\r" or key == "\n":
+                break
+    sck = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    sck.connect(("localhost", port))
+
+
+def get_windows_signal():
+    """Using socket as a replacement for stdin on Windows."""
+    (sig, sig_port) = create_socket(8000, 9000)
+    threading.Thread(target=capture_enter, args=(sig_port,)).start()
+    return sig
+
+
+try:
+    httpd_fileno = httpd.fileno()
+    sig = get_windows_signal() if is_windows() else sys.stdin
+    sig_fileno = sig.fileno()
+    while True:
+        ready_read = select.select([sig_fileno, httpd_fileno], [], [])
+        if ready_read[0][0] != httpd_fileno:
             print("Shutting down.")
-            # Consume the newline so it doesn't get printed by the caller
-            sys.stdin.readline()
+
+            # On windows the newline has already been consumed by the capture_enter function.
+            if not is_windows():
+                # Consume the newline so it doesn't get printed by the caller
+                sys.stdin.readline()
             break
         else:
             httpd.handle_request()
@@ -1731,5 +1627,8 @@ except KeyboardInterrupt:
     print("\nShutting down.")
 
 # Clean up temporary file
-f.close()
+# If on Windows, the file already closed
+if not is_windows():
+    f.close()
+
 thread.join()
